/*
   wmfw.c
 
  In enlish:
    wmfw shows weather from chosen finnish city in WindowMaker-dockapp from www.ilmatieteenlaitos.fi.

    This application uses docklib-0.4(libdock 0.4) and requires it for compilation. 
    Maeby docklib-0.2 is suitable too. Texts are in finnish so I thing it's not usable like this 
    for people who are not finnish, but it's possible to make your own. Take parts from here 
    and make your own wmXX.
  
  SUOMEKSI:
    WindowMaker-dockapp suomalaisille kaupungeille ilmatieteenlaistoksen-webbi sivuilta.
    wmfw nytt tietyilt halutuilta paikkakunnista lmptilan, kosteus prosentin jne.,
    mit nyt sattuu mittaus paikasta lytymn.

    Ohjelma kytt docklib-0.4(libdock 0.4) ja knnettess vaati sen asennettuna.
    Epilen ett ohjelma kntyy mys docklib-0.2:n avulla. Katso README-tiedosto ohjelman 
    kytt koskeviin ohjeisiin ja COPYING-tiedosto lisenssisseihin.

  Copyright (c) 2004 Joni Valtanen 
    licensed under GNU General Public License. See COPYING for details. 

  Copyright (c) 1999 Alfredo K. Kojima
    Parts of the program are from docklib-0.2 example. ( basic structure )

  Copyright (c) 1999  Sam Hawker
    font_init()-function and other font related stuff from wmmount ( Yes I'm lazy )   

  See the README and COPYING for detailed licences.

*/


#include <dockapp.h>
#include <unistd.h>
#include <math.h>
#include <string.h>

#include "engine.h"

#define WMFONT "-*-fixed-medium-r-*-*-8-*-*-*-*-*-*-*"
#define WMWH 58
#define SLEEPTIME 10000

/* global var*/
extern struct Info info;

char *wmfont=NULL;
Font fname;
int fx,fy;

GC gc;
Pixmap pixmap;
char buf[20][50];

time_t updateTime=0; /* seconds */
int waitTime = 30; /* minutes */

short scr=0;
short lines=0;

char *displayName = "";
char *cityName = "";

static DAProgramOption options[] = {
  {"-time", NULL, "update time in minutes", DOInteger, False, {&waitTime}}, 
  {"-city", NULL, "city of Finland to display", DOString, False, {&cityName}},
    {"-display", NULL, "display to use", DOString, False, {&displayName}}
};

void buttonrelease(XButtonEvent *xev);
void buttonpress(XButtonEvent *xev);
void font_init(void);
void setTimes(void);

void firstscr(void);
void otherscr(void);

/*
  int main...
*/
int main(int argc, char *argv[])
{
   
  int i,j,k;

  DAParseArguments(argc, argv, options,
		   sizeof(options)/sizeof(DAProgramOption),
		   "WMFW dockapp", "WMFW 0.5.2");

  DAInitialize(displayName, "WMFW", WMWH, WMWH, argc, argv);
  pixmap = DAMakePixmap();
  gc = DefaultGC(DADisplay, DefaultScreen(DADisplay));
  font_init();

  XSetFont(DADisplay, gc, fname);

  DAShow();

  while ( 1 ) {
    XEvent ev;
    i = j = k = 0;

    if(( updateTime - time(NULL) < 0))
      {

	getWeather(cityName);
	setTimes();

	sprintf(buf[k++], "%s", info.place);
	sprintf(buf[k++], "%s", info.date);
	sprintf(buf[k++], "%s", info.clock);

	for(i=0; info.valname[i]!=NULL; i++)
	  {
#ifdef DEBUG
	    printf("%s\n",info.valname[i]);
#endif
	    for(j=0; info.value[i][j] != '<' && info.value[i][j] != 0; j++);
	    info.value[i][j] = 0;
	    sprintf(buf[k++], "%s:",info.valname[i] );
	    sprintf(buf[k++], "%s %s",info.value[i],magnitude(info.valname[i]) );
	  }
        lines = i*2;

      }

    XSetForeground(DADisplay, gc, 
		   BlackPixel(DADisplay, DefaultScreen(DADisplay)));
    XFillRectangle(DADisplay, pixmap, gc, 0, 0, WMWH, WMWH);

    XSetForeground(DADisplay, gc, 
		   WhitePixel(DADisplay, DefaultScreen(DADisplay)));

    if(scr == 0)
      firstscr();
    else
      otherscr();

    DASetPixmap(pixmap);

    XSelectInput(DADisplay, DAWindow, ButtonPressMask | ButtonReleaseMask | ExposureMask);

    /* handle all pending X events */
    while (XPending(DADisplay)) {
      XNextEvent(DADisplay, &ev);
      switch(ev.type) {
      case ButtonPress:
        buttonpress(&ev.xbutton);
	break;
      case ButtonRelease:
	buttonrelease(&ev.xbutton);
	break;}
      DAProcessEvent(&ev);
    }

    usleep( SLEEPTIME );
  }
  return 0;
}

/*
  void buttonrelease(XButtonEvent *xev)

*/
void buttonrelease(XButtonEvent *xev)
{

  short pages=0;

  if( (lines-2)%4 )
    pages = (lines-2)/4 + 1;
  else
    pages = (lines-2)/4;
 
  scr++;
  if(scr > pages)
    scr=0;

  XSetForeground(DADisplay, gc, 
		 BlackPixel(DADisplay, DefaultScreen(DADisplay)));
  XFillRectangle(DADisplay, pixmap, gc, 0, 0, 58, 58);

  XSetForeground(DADisplay, gc, 
		 WhitePixel(DADisplay, DefaultScreen(DADisplay)));

  if(scr == 0)
    firstscr();
  else
    otherscr();

}

/*
  void buttonpress(XButtonEvent *xev)

*/
void buttonpress(XButtonEvent *xev)
{

 
}

/*
  void firstscr(void)
  draws first screen

  INPUT: -
  OUT:	-
*/
void firstscr(void)
{
  int i;

  for(i=0; i<3; i++)
    XDrawString(DADisplay, pixmap, gc, 1, ((1+i)*(10)), buf[i], strlen(buf[i]));
  for(i=3; i<5; i++)
    XDrawString(DADisplay, pixmap, gc, 1, (45+((i-3)*10)), buf[i], strlen(buf[i]));
}

/*
  void otherscr(void)
  draw other screens than first

  INPUT: -
  OUT:	-
*/
void otherscr(void)
{
  int i;

  /*scr=1 bufferlines 5-8 to draw*/
  /*scr=2 9-12 to draw*/

  /* Draw text to dock */
  XDrawString(DADisplay, pixmap, gc, 1, 10, buf[0], strlen(buf[0]));
  for(i=1; i<5; i++)
    XDrawString(DADisplay, pixmap, gc, 1, (((i)*(10))+15), buf[scr*4+i], strlen(buf[scr*4+i]));
}


/*
  void font_init(void)
  inits fonts

  INPUT: -
  OUT: -
*/
void font_init(void)
{
  /* Initialize fonts
   */
  XFontStruct *xfs;
  char *font;

  font=wmfont;
  if(font==NULL)
    font=WMFONT;
  xfs=XLoadQueryFont(DADisplay, WMFONT);
  
  fname=xfs->fid;
  fx=6-xfs->min_bounds.lbearing;
  fy+=xfs->ascent;
  XFreeFontInfo(NULL, xfs, 1);

}

/*
  void setTimes(void)
  Set updatetime in seconds

  INPUT: -
  OUT: -
*/
void setTimes(void)
{
  updateTime = time(NULL)+(60*waitTime);
}
